@@ -143,7 +143,6 @@ ComputerModel::ComputerModel(QObject *parent) : ForwardingModel(parent)
 , m_appletInterface(nullptr)
 {
     connect(m_systemAppsModel, &FavoritesModel::favoritesChanged, this, &ComputerModel::systemApplicationsChanged);
-    m_systemAppsModel->setFavorites(QStringList() << "systemsettings.desktop");
 
     m_concatProxy->addSourceModel(m_runCommandModel);
     m_concatProxy->addSourceModel(m_systemAppsModel);
@@ -220,6 +219,8 @@ QVariant ComputerModel::data(const QModelIndex &index, int role) const
 
                 if (access) {
                     return access->filePath();
+                } else {
+                    return QString();
                 }
             } else {
                 const QUrl &url = m_filteredPlacesModel->url(sourceIndex);
