@@ -32,6 +32,7 @@
 #include <QDrag>
 #include <QImage>
 #include <QItemSelectionModel>
+#include <QMenu>
 #include <QMimeData>
 #include <QMimeDatabase>
 #include <QPainter>
@@ -40,12 +41,11 @@
 #include <QQuickWindow>
 #include <qplatformdefs.h>
 
-#include "internallibkonq/konq_popupmenu.h"
 #include <KDirWatch>
 #include <KIO/DropJob>
 #include <KAuthorized>
-#include <KBookmarkManager>
 #include <KConfigGroup>
+#include <KFileCopyToMenu>
 #include <KFileItemActions>
 #include <KFileItemListProperties>
 #include <KNewFileMenu>
@@ -55,9 +55,12 @@
 #include <KIO/JobUiDelegate>
 #include <KIO/Paste>
 #include <KIO/PasteJob>
+#include <KIO/RestoreJob>
 #include <KLocalizedString>
+#include <KPropertiesDialog>
 #include <KSharedConfig>
 #include <KShell>
+#include <kio_version.h>
 
 #include <KDesktopFile>
 #include <KDirModel>
@@ -148,8 +151,10 @@ QHash< int, QByteArray > FolderModel::staticRoleNames()
     roleNames[Qt::DisplayRole] = "display";
     roleNames[Qt::DecorationRole] = "decoration";
     roleNames[BlankRole] = "blank";
+    roleNames[OverlaysRole] = "overlays";
     roleNames[SelectedRole] = "selected";
     roleNames[IsDirRole] = "isDir";
+    roleNames[IsLinkRole] = "isLink";
     roleNames[UrlRole] = "url";
     roleNames[LinkDestinationUrl] = "linkDestinationUrl";
     roleNames[SizeRole] = "size";
@@ -169,7 +174,6 @@ void FolderModel::setUrl(const QString& url)
 
     if (url == m_url) {
         m_dirModel->dirLister()->updateDirectory(resolvedUrl);
-
         return;
     }
 
@@ -190,7 +194,7 @@ void FolderModel::setUrl(const QString& url)
         delete m_dirWatch;
     }
 
-    if (resolvedUrl.isLocalFile()) {
+    if (resolvedUrl.isValid()) {
         m_dirWatch = new KDirWatch(this);
         connect(m_dirWatch, &KDirWatch::created, this, &FolderModel::iconNameChanged);
         connect(m_dirWatch, &KDirWatch::dirty, this, &FolderModel::iconNameChanged);
@@ -487,10 +491,20 @@ void FolderModel::cd(int row)
         return;
     }
 
-    KFileItem item = itemForIndex(index(row, 0));
+    const QModelIndex idx = index(row, 0);
+    bool isDir = data(idx, IsDirRole).toBool();
 
-    if (item.isDir()) {
-        setUrl(item.url().toString());
+    if (isDir) {
+        const KFileItem  item = itemForIndex(idx);
+        if (m_parseDesktopFiles && item.isDesktopFile()) {
+            const KDesktopFile file(item.targetUrl().path());
+            if (file.readType() == QLatin1String("Link")) {
+                setUrl(file.readUrl());
+            }
+        }
+        else {
+            setUrl(item.url().toString());
+        }
     }
 }
 
@@ -512,6 +526,22 @@ void FolderModel::run(int row)
     new KRun(url, 0);
 }
 
+void FolderModel::runSelected()
+{
+    if (!m_selectionModel->hasSelection()) {
+        return;
+    }
+
+    bool unarySelection = (m_selectionModel->selectedIndexes().count() == 1);
+
+    foreach (const QModelIndex &index, m_selectionModel->selectedIndexes()) {
+        // Skip over directories.
+        if (!unarySelection && !index.data(IsDirRole).toBool()) {
+            run(index.row());
+        }
+    }
+}
+
 void FolderModel::rename(int row, const QString& name)
 {
     if (row < 0) {
@@ -723,6 +753,12 @@ void FolderModel::addDragImage(QDrag *drag, int x, int y)
 
 void FolderModel::dragSelected(int x, int y)
 {
+    if (m_dragInProgress) {
+        return;
+    }
+
+    m_dragInProgress = true;
+
     // Avoid starting a drag synchronously in a mouse handler or interferes with
     // child event filtering in parent items (and thus e.g. press-and-hold hand-
     // ling in a containment).
@@ -734,6 +770,7 @@ void FolderModel::dragSelected(int x, int y)
 void FolderModel::dragSelectedInternal(int x, int y)
 {
     if (!m_viewAdapter || !m_selectionModel->hasSelection()) {
+        m_dragInProgress = false;
         return;
     }
 
@@ -760,7 +797,6 @@ void FolderModel::dragSelectedInternal(int x, int y)
     drag->setMimeData(m_dirModel->mimeData(sourceDragIndexes));
 
     item->grabMouse();
-    m_dragInProgress = true;
     drag->exec(supportedDragActions());
     m_dragInProgress = false;
     item->ungrabMouse();
@@ -933,6 +969,9 @@ QVariant FolderModel::data(const QModelIndex& index, int role) const
 
     if (role == BlankRole) {
         return m_dragIndexes.contains(index);
+    } else if (role == OverlaysRole) {
+        const KFileItem item = itemForIndex(index);
+        return item.overlays();
     } else if (role == SelectedRole) {
         return m_selectionModel->isSelected(index);
     } else if (role == IsDirRole) {
@@ -943,6 +982,9 @@ QVariant FolderModel::data(const QModelIndex& index, int role) const
         } else {
             return isDir(mapToSource(index), m_dirModel);
         }
+    } else if (role == IsLinkRole) {
+        const KFileItem item = itemForIndex(index);
+        return item.isLink();
     } else if (role == UrlRole) {
         return itemForIndex(index).url();
     } else if (role == LinkDestinationUrl) {
@@ -1193,9 +1235,16 @@ void FolderModel::createActions()
     QAction *emptyTrash = new QAction(QIcon::fromTheme(QStringLiteral("trash-empty")), i18n("&Empty Trash Bin"), this);
     connect(emptyTrash, &QAction::triggered, this, &FolderModel::emptyTrashBin);
 
+    QAction *restoreFromTrash = new QAction(i18nc("Restore from trash", "Restore"), this);
+    connect(restoreFromTrash, &QAction::triggered, this, &FolderModel::restoreSelectedFromTrash);
+
     QAction *del = new QAction(QIcon::fromTheme(QStringLiteral("edit-delete")), i18n("&Delete"), this);
     connect(del, &QAction::triggered, this, &FolderModel::deleteSelected);
 
+    QAction *actOpen = new QAction(QIcon::fromTheme(QStringLiteral("window-new")), i18n("&Open"), this);
+    connect(actOpen, &QAction::triggered, this, &FolderModel::openSelected);
+
+    m_actionCollection.addAction(QStringLiteral("open"), actOpen);
     m_actionCollection.addAction(QStringLiteral("cut"), cut);
     m_actionCollection.addAction(QStringLiteral("undo"), undo);
     m_actionCollection.addAction(QStringLiteral("copy"), copy);
@@ -1206,10 +1255,13 @@ void FolderModel::createActions()
     m_actionCollection.addAction(QStringLiteral("rename"), rename);
     m_actionCollection.addAction(QStringLiteral("trash"), trash);
     m_actionCollection.addAction(QStringLiteral("del"), del);
+    m_actionCollection.addAction(QStringLiteral("restoreFromTrash"), restoreFromTrash);
     m_actionCollection.addAction(QStringLiteral("emptyTrash"), emptyTrash);
 
     m_newMenu = new KNewFileMenu(&m_actionCollection, QStringLiteral("newMenu"), QApplication::desktop());
     m_newMenu->setModal(false);
+
+    m_copyToMenu = new KFileCopyToMenu(Q_NULLPTR);
 }
 
 QAction* FolderModel::action(const QString &name) const
@@ -1229,10 +1281,12 @@ void FolderModel::updateActions()
         m_newMenu->setPopupFiles(m_dirModel->dirLister()->url());
     }
 
+    const bool isTrash = (resolvedUrl().scheme() == QLatin1String("trash"));
+
     QAction *emptyTrash = m_actionCollection.action(QStringLiteral("emptyTrash"));
 
     if (emptyTrash) {
-        if (resolvedUrl() == QUrl(QStringLiteral("trash:/"))) {
+        if (isTrash) {
             emptyTrash->setVisible(true);
             KConfig trashConfig(QStringLiteral("trashrc"), KConfig::SimpleConfig);
             emptyTrash->setEnabled(!trashConfig.group("Status").readEntry("Empty", true));
@@ -1241,6 +1295,10 @@ void FolderModel::updateActions()
         }
     }
 
+    if (QAction *restoreFromTrash = m_actionCollection.action(QStringLiteral("restoreFromTrash"))) {
+        restoreFromTrash->setVisible(isTrash);
+    }
+
     QAction *paste = m_actionCollection.action(QStringLiteral("paste"));
 
     if (paste) {
@@ -1276,10 +1334,13 @@ void FolderModel::openContextMenu()
 
     updateActions();
 
-    QMenu *menu = 0;
+    QMenu *menu = new QMenu();
+    if (!m_fileItemActions) {
+        m_fileItemActions = new KFileItemActions(this);
+        m_fileItemActions->setParentWidget(QApplication::desktop());
+    }
 
     if (indexes.isEmpty()) {
-        menu = new QMenu();
         menu->addAction(m_actionCollection.action(QStringLiteral("newMenu")));
         menu->addSeparator();
         menu->addAction(m_actionCollection.action(QStringLiteral("paste")));
@@ -1288,26 +1349,27 @@ void FolderModel::openContextMenu()
         menu->addAction(m_actionCollection.action(QStringLiteral("emptyTrash")));
         menu->addSeparator();
 
-        if (!m_fileItemActions) {
-            m_fileItemActions = new KFileItemActions(this);
-        }
-
-        KFileItemListProperties itemList(KFileItemList() << m_dirModel->dirLister()->rootItem());
-        m_fileItemActions->setItemListProperties(itemList);
+        KFileItemListProperties itemProperties(KFileItemList() << m_dirModel->dirLister()->rootItem());
+        m_fileItemActions->setItemListProperties(itemProperties);
 
         menu->addAction(m_fileItemActions->preferredOpenWithAction(QString()));
     } else {
         KFileItemList items;
+        QList<QUrl> urls;
         bool hasRemoteFiles = false;
         bool isTrashLink = false;
 
+        items.reserve(indexes.count());
+        urls.reserve(indexes.count());
         foreach (const QModelIndex &index, indexes) {
             KFileItem item = itemForIndex(index);
             if (!item.isNull()) {
                 hasRemoteFiles |= item.localPath().isEmpty();
                 items.append(item);
+                urls.append(item.url());
             }
         }
+        KFileItemListProperties itemProperties(items);
 
         // Check if we're showing the menu for the trash link
         if (items.count() == 1 && items.at(0).isDesktopFile()) {
@@ -1317,8 +1379,20 @@ void FolderModel::openContextMenu()
             }
         }
 
-        QList<QAction*> editActions;
-        editActions.append(m_actionCollection.action(QStringLiteral("rename")));
+        // Start adding the actions:
+        menu->addAction(m_actionCollection.action(QStringLiteral("open")));
+        menu->addSeparator();
+        if (itemProperties.supportsDeleting()) {
+            menu->addAction(m_actionCollection.action(QStringLiteral("cut")));
+        }
+        menu->addAction(m_actionCollection.action(QStringLiteral("copy")));
+
+        if (itemProperties.isDirectory() && itemProperties.supportsWriting()) {
+            menu->addAction(m_actionCollection.action(QStringLiteral("pasteto")));
+        }
+
+        menu->addAction(m_actionCollection.action(QStringLiteral("rename")));
+        menu->addAction(m_actionCollection.action(QStringLiteral("restoreFromTrash")));
 
         KSharedConfig::Ptr globalConfig = KSharedConfig::openConfig(QStringLiteral("kdeglobals"), KConfig::NoGlobals);
         KConfigGroup cg(globalConfig, "KDE");
@@ -1327,31 +1401,49 @@ void FolderModel::openContextMenu()
         // Don't add the "Move to Trash" action if we're showing the menu for the trash link
         if (!isTrashLink) {
             if (!hasRemoteFiles) {
-                editActions.append(m_actionCollection.action(QStringLiteral("trash")));
+                menu->addAction(m_actionCollection.action(QStringLiteral("trash")));
             } else {
                 showDeleteCommand = true;
             }
         }
         if (showDeleteCommand) {
-            editActions.append(m_actionCollection.action(QStringLiteral("del")));
+            menu->addAction(m_actionCollection.action(QStringLiteral("del")));
         }
 
-        KonqPopupMenu::ActionGroupMap actionGroups;
-        actionGroups.insert(KonqPopupMenu::EditActions, editActions);
+        // "Open with" actions
+        m_fileItemActions->setItemListProperties(itemProperties);
+        m_fileItemActions->addOpenWithActionsTo(menu);
+        // Service actions
+        m_fileItemActions->addServiceActionsTo(menu);
+        menu->addSeparator();
+        // Plugin actions
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 27, 0)
+        m_fileItemActions->addPluginActionsTo(menu);
+#endif
 
-        KonqPopupMenu::Flags flags = KonqPopupMenu::ShowProperties;
-        flags |= KonqPopupMenu::ShowUrlOperations;
-        flags |= KonqPopupMenu::ShowNewWindow;
+        // Copy To, Move To
+        KSharedConfig::Ptr dolphin = KSharedConfig::openConfig(QStringLiteral("dolphinrc"));
+        if (KConfigGroup(dolphin, "General").readEntry("ShowCopyMoveMenu", false)) {
+            m_copyToMenu->setUrls(urls);
+            m_copyToMenu->setReadOnly(!itemProperties.supportsMoving());
+            m_copyToMenu->addActionsTo(menu);
+            menu->addSeparator();
+        }
+
+        // Properties
+        if (KPropertiesDialog::canDisplay(items)) {
+            QAction *act = new QAction(menu);
+            act->setText(i18n("&Properties"));
+            QObject::connect(act, &QAction::triggered, [this, items]() {
+                    KPropertiesDialog::showDialog(items, Q_NULLPTR, false /*non modal*/);
+            });
+            menu->addAction(act);
+        }
 
-        KonqPopupMenu *popupMenu = new KonqPopupMenu(items, m_dirModel->dirLister()->url(), m_actionCollection, flags, QApplication::desktop());
-        popupMenu->setNewFileMenu(m_newMenu);
-        popupMenu->setBookmarkManager(KBookmarkManager::userBookmarksManager());
-        popupMenu->setActionGroups(actionGroups);
-        connect(popupMenu, &QMenu::aboutToHide, [popupMenu]() { popupMenu->deleteLater(); });
-        menu = popupMenu;
     }
 
     menu->popup(QCursor::pos());
+    connect(menu, &QMenu::aboutToHide, [menu]() { menu->deleteLater(); });
 }
 
 void FolderModel::linkHere(const QUrl &sourceUrl)
@@ -1448,6 +1540,18 @@ void FolderModel::deleteSelected()
     }
 }
 
+void FolderModel::openSelected()
+{
+    if (!m_selectionModel->hasSelection()) {
+        return;
+    }
+
+    const QList<QUrl> urls = selectedUrls(false);
+    for (const QUrl &url : urls) {
+        (void) new KRun(url, Q_NULLPTR);
+    }
+}
+
 void FolderModel::emptyTrashBin()
 {
     KIO::JobUiDelegate uiDelegate;
@@ -1458,6 +1562,18 @@ void FolderModel::emptyTrashBin()
     }
 }
 
+void FolderModel::restoreSelectedFromTrash()
+{
+    if (!m_selectionModel->hasSelection()) {
+        return;
+    }
+
+    const auto &urls = selectedUrls(true);
+
+    KIO::RestoreJob *job = KIO::restoreFromTrash(urls);
+    job->ui()->setAutoErrorHandlingEnabled(true);
+}
+
 void FolderModel::undoTextChanged(const QString &text)
 {
     if (QAction *action = m_actionCollection.action(QStringLiteral("undo"))) {
