@@ -21,6 +21,7 @@
 #include "../krdb/krdb.h"
 #include "../cursortheme/xcursor/xcursortheme.h"
 #include "config-kcm.h"
+#include "config-workspace.h"
 #include <klauncher_iface.h>
 
 #include <KPluginFactory>
@@ -68,6 +69,7 @@ KCMLookandFeel::KCMLookandFeel(QObject* parent, const QVariantList& args)
     , m_applyWindowSwitcher(true)
     , m_applyDesktopSwitcher(true)
     , m_resetDefaultLayout(false)
+    , m_applyWindowDecoration(true)
 {
     //This flag seems to be needed in order for QQuickWidget to work
     //see https://bugreports.qt-project.org/browse/QTBUG-40765
@@ -241,7 +243,6 @@ void KCMLookandFeel::load()
 
         m_model->appendRow(row);
     }
-    setNeedsSave(false);
 }
 
 
@@ -296,8 +297,27 @@ void KCMLookandFeel::save()
                     colorScheme.replace(offset, fixer.matchedLength(), fixer.cap(1).toUpper());
                 }
                 colorScheme.replace(0, 1, colorScheme.at(0).toUpper());
-                QString src = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "color-schemes/" +  colorScheme + ".colors");
-                setColors(colorScheme, src);
+
+                //NOTE: why this loop trough all the scheme files?
+                //the scheme theme name is an heuristic, there is no plugin metadata whatsoever.
+                //is based on the file name stripped from weird characters or the
+                //eventual id- prefix store.kde.org puts, so we can just find a
+                //theme that ends as the specified name
+                bool schemeFound = false;
+                const QStringList schemeDirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("color-schemes"), QStandardPaths::LocateDirectory);
+                for (const QString &dir : schemeDirs) {
+                    const QStringList fileNames = QDir(dir).entryList(QStringList()<<QStringLiteral("*.colors"));
+                    for (const QString &file : fileNames) {
+                        if (file.endsWith(colorScheme + ".colors")) {
+                            setColors(colorScheme, dir + QChar('/') + file);
+                            schemeFound = true;
+                            break;
+                        }
+                    }
+                    if (schemeFound) {
+                        break;
+                    }
+                }
             }
         }
 
@@ -330,6 +350,24 @@ void KCMLookandFeel::save()
             cg = KConfigGroup(&cg, "DesktopSwitcher");
             setDesktopSwitcher(cg.readEntry("LayoutName", QString()));
         }
+
+        if (m_applyWindowDecoration) {
+            cg = KConfigGroup(conf, "kwinrc");
+            cg = KConfigGroup(&cg, "org.kde.kdecoration2");
+#if HAVE_BREEZE_DECO
+            setWindowDecoration(cg.readEntry("library", QStringLiteral(BREEZE_KDECORATION_PLUGIN_ID)), cg.readEntry("theme", QString()));
+#else
+            setWindowDecoration(cg.readEntry("library", QStringLiteral("org.kde.kwin.aurorae")), cg.readEntry("theme", QString()));
+#endif
+        }
+
+        // Reload KWin if something changed, but only once.
+        if (m_applyWindowSwitcher || m_applyDesktopSwitcher || m_applyWindowDecoration) {
+            QDBusMessage message = QDBusMessage::createSignal(QStringLiteral("/KWin"),
+                                                    QStringLiteral("org.kde.KWin"),
+                                                    QStringLiteral("reloadConfig"));
+            QDBusConnection::sessionBus().send(message);
+        }
     }
 
     //TODO: option to enable/disable apply? they don't seem required by UI design
@@ -596,11 +634,6 @@ void KCMLookandFeel::setWindowSwitcher(const QString &theme)
     KConfigGroup cg(&config, "TabBox");
     cg.writeEntry("LayoutName", theme);
     cg.sync();
-    // Reload KWin.
-    QDBusMessage message = QDBusMessage::createSignal(QStringLiteral("/KWin"),
-                                                      QStringLiteral("org.kde.KWin"),
-                                                      QStringLiteral("reloadConfig"));
-    QDBusConnection::sessionBus().send(message);
 }
 
 void KCMLookandFeel::setDesktopSwitcher(const QString &theme)
@@ -614,6 +647,20 @@ void KCMLookandFeel::setDesktopSwitcher(const QString &theme)
     cg.writeEntry("DesktopLayout", theme);
     cg.writeEntry("DesktopListLayout", theme);
     cg.sync();
+}
+
+void KCMLookandFeel::setWindowDecoration(const QString &library, const QString &theme)
+{
+    if (library.isEmpty()) {
+        return;
+    }
+
+    KConfig config(QStringLiteral("kwinrc"));
+    KConfigGroup cg(&config, "org.kde.kdecoration2");
+    cg.writeEntry("library", library);
+    cg.writeEntry("theme", theme);
+    
+    cg.sync();
     // Reload KWin.
     QDBusMessage message = QDBusMessage::createSignal(QStringLiteral("/KWin"),
                                                       QStringLiteral("org.kde.KWin"),
@@ -621,7 +668,6 @@ void KCMLookandFeel::setDesktopSwitcher(const QString &theme)
     QDBusConnection::sessionBus().send(message);
 }
 
-
 void KCMLookandFeel::setApplyColors(bool apply)
 {
     if (m_applyColors == apply) {
